// CAR.H - class and structure declarations for RARS (Robot Auto Racing Sim)
// by Mitchell E. Timin, State College, PA
// This version is for Borland C++, version 3.1, and is for DOS
// This is part of version 0.39 of RARS (Robot Auto Racing Simulation)
// ver. 0.1 release January 12, 1995
// ver. 0.2 1/23/95
// ver. 0.3 2/7/95
// ver. 0.39 3/6/95

#include "gi.h"

const int MAXCARS = 16;        // cannot have more cars than this on the track
const double MPH_FPS = 3600.0 / 5280.0;    // mph per fps
const double CARLEN = 30;           // feet (Big Car! - to see it better)
const double CARWID = 9;            // feet

struct con_vec { double alpha, vc; };  // control vector, steering & throttle

// relative position and velocity vectors of a car in front
// These are wrt a non-rotating frame of reference centered on
// the car behind.  The y axis is in the direction the car is pointing.
struct rel_state {
   double rel_x;     // how far to your right is the car ahead?
   double rel_y;     // how far in front of you?
   double rel_xdot;  // how fast cutting accross from left-to-right?
   double rel_ydot;  // how fast is he getting away from you?
   int who;          // an identifier for that car.
};

// This struct is calculated by observe() and used by control()
struct situation {       // a car's local situation as seen by the driver
   double cur_rad;       // radius of inner wall of curve (0 means straight)
   double cur_len;       // length of current track segment (angle if curve)
   double to_lft;        // distance to left wall
   double to_rgt;        // distance to right wall
   double to_end;        // how far to end of current track seg. (angle or feet)
   double v;             // the speed of the car, feet per second
   double vn;            // component of v perpendicular to track direction
   double nex_len;       // length of the next track segment (angle if curve)
   double nex_rad;       // radius of inner wall of next segment (or 0)
   double after_rad;     // radius of the segment after that one. (or 0)
   double power_req;     // ratio: power requested by driver to maximum power
   int dead_ahead;       // set when there is a car dead ahead, else 0
   int backward;         // set if cars motion is opposed to track direction
   int starting;           // if not zero, robot knows to initialize data
   rel_state* nearby;      // relative states of three cars in front of you
   void* data_ptr;         // pointer to driver's scratchpad RAM area
};

// Each car is an instance of this class:
class Car {
   int nose_color, tail_color;   // the car's colors
   double x, y, xdot, ydot, ang, adot;  // current state of car (feet, seconds)
   double prex, prey, prang;            // previous state drawn on screen
   int seg_id;                          // current track segment index
   int lap_flag;         // changes from 0 to 1 on each crossing of finish line
   int laps;                            // laps completed
   double to_end;                       // same as in s.to_end, above
   int out;                             // out of race when set
   double cen_a, tan_a;                 // centripetal, tangential acceleration
   double alpha, vc;        // wheel angle of attack and wheel command velocity
   double prev_alpha;       // previous value of alpha
   int which;                           // which car (index into pcar[] array)
   int offroad, veryoffroad;            // flags, set if off the track
   double power_req;          // power requested by driver, divided by PwrMax
   int dead_ahead;            // set iff there is another car dead ahead
   unsigned int damage;       // accumulated damage units (out of race 10000)
   unsigned int init_flag;       // for use only by control program (driver)
   double start_time;            // time instant of crossing starting line
   int starting;                 // if not zero, robot knows to initialize
   con_vec (*cntrl)(situation);  // cntrl is pointer to the car's driver program
   void* data_ptr;               // pointer to driver's scratchpad RAM area
public:
   double speed_avg, speed_max;         // average and maximum speed
   Car(colors, double, double, double, int);    // constructor
   ~Car(void);                                  // Destructor
   situation observe(rel_state*);  // computes the car's local situation
   void control(situation);  // the "driver" - actual code selected by cntrl[]
   void move_car();          // simulates the physics of car, track, control
   void draw_car(void);       // erases and redraws the car
   inline int where_is_it(int*, double*);    // returns laps and other locators
   friend int farther(Car*, Car*);  // which has gone further of the two cars
};

// function to get a car out of a crash or pileup or spin-out:
extern int stuck(int, double, double, double, double, double*, double*);


